from http import HTTPStatus
from typing import Any, Dict, Optional, Union

import httpx

from ... import errors
from ...client import Client
from ...types import UNSET, Response, Unset


def _get_kwargs(
    id: str,
    media_source_id: str,
    index: int,
    start_position_ticks: int,
    format_: str,
    *,
    client: Client,
    end_position_ticks: Union[Unset, None, int] = UNSET,
    copy_timestamps: Union[Unset, None, bool] = UNSET,
) -> Dict[str, Any]:
    url = "{}/Videos/{Id}/{MediaSourceId}/Subtitles/{Index}/{StartPositionTicks}/Stream.{Format}".format(
        client.base_url,
        Id=id,
        MediaSourceId=media_source_id,
        Index=index,
        StartPositionTicks=start_position_ticks,
        Format=format_,
    )

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    params: Dict[str, Any] = {}
    params["EndPositionTicks"] = end_position_ticks

    params["CopyTimestamps"] = copy_timestamps

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "method": "get",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "follow_redirects": client.follow_redirects,
        "params": params,
    }


def _parse_response(*, client: Client, response: httpx.Response) -> Optional[Any]:
    if response.status_code == HTTPStatus.OK:
        return None
    if response.status_code == HTTPStatus.BAD_REQUEST:
        return None
    if response.status_code == HTTPStatus.NOT_FOUND:
        return None
    if response.status_code == HTTPStatus.INTERNAL_SERVER_ERROR:
        return None
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client: Client, response: httpx.Response) -> Response[Any]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    id: str,
    media_source_id: str,
    index: int,
    start_position_ticks: int,
    format_: str,
    *,
    client: Client,
    end_position_ticks: Union[Unset, None, int] = UNSET,
    copy_timestamps: Union[Unset, None, bool] = UNSET,
) -> Response[Any]:
    """Gets subtitles in a specified format.

     No authentication required

    Args:
        id (str):
        media_source_id (str):
        index (int):
        start_position_ticks (int):
        format_ (str):
        end_position_ticks (Union[Unset, None, int]):
        copy_timestamps (Union[Unset, None, bool]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        id=id,
        media_source_id=media_source_id,
        index=index,
        start_position_ticks=start_position_ticks,
        format_=format_,
        client=client,
        end_position_ticks=end_position_ticks,
        copy_timestamps=copy_timestamps,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(client=client, response=response)


async def asyncio_detailed(
    id: str,
    media_source_id: str,
    index: int,
    start_position_ticks: int,
    format_: str,
    *,
    client: Client,
    end_position_ticks: Union[Unset, None, int] = UNSET,
    copy_timestamps: Union[Unset, None, bool] = UNSET,
) -> Response[Any]:
    """Gets subtitles in a specified format.

     No authentication required

    Args:
        id (str):
        media_source_id (str):
        index (int):
        start_position_ticks (int):
        format_ (str):
        end_position_ticks (Union[Unset, None, int]):
        copy_timestamps (Union[Unset, None, bool]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        id=id,
        media_source_id=media_source_id,
        index=index,
        start_position_ticks=start_position_ticks,
        format_=format_,
        client=client,
        end_position_ticks=end_position_ticks,
        copy_timestamps=copy_timestamps,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(client=client, response=response)
